function	 [B2,A2]=l2minity(Htf,w,nb,na,tol,maxIter,D0,errTol,show)
%	function [B2,A2]=l2minity(Htf,w,nb,na,tol,maxIter,D0,errTol)
%	Given an order specification this function iterates trying to  
%	approximatethe htfi given
%
%	tol is the stopping criteria for the iteration (see code)
%	B0 is the initial guess for B2
%	errTol is an optinal stopping criteria (stops if error below errTol dB)

if nargin<4
	disp('This function must have at least four parameters. Try: help l2minit');
end;	

Htf=Htf(:); w=w(:); % Putting the entries in vector form (just to be on the safe side).

if nargin<9
	show=1;
end;	

if nargin<8
	errTol=1E-3;
end;	

if nargin<7
	D0=speye(length(w));
end;	

if nargin<6
	maxIter=100;
end;	

if nargin<5
	tol=1E-2;
end;	

omega=w(:); 

%
%	Here we impose no constraints so lets begin.
%	The first approximation doesn't use scaling of any kind (we assume max(w)=1);

imH=1./abs(Htf);

mD2=imH;
mmm=max(abs(mD2));
mD2(1)=1000*mmm(1);

A1=[ones(length(w),1) powerseq(i*w,nb) -diag(Htf)*powerseq(i*w,na)];
b1=Htf;

A=[diag(mD2)*real(A1); diag(mD2)*imag(A1)];
B=[diag(mD2)*real(b1); diag(mD2)*imag(b1)];

[Q,R]=qr(A,0);
BB=Q'*B;

C=R\BB;

B2=C(nb+1:-1:1);
A2=[C(nb+na+1:-1:nb+2); 1];

H2=freqs(B2,A2,w);
E2=Htf./H2;

m1=max(abs(20*log10(abs(E2))));
m2=mean(abs(20*log10(abs(E2))));

oldA1=A1;
oldb1=b1;
oldm1=m1;
peso=1.5;

k=1;
converged=0;

D2=ones(size(w)); N2=ones(size(w));

PesosDeErro=ones(size(imH));

m1_acum = m1;
m2_acum = m2;

while (k<maxIter)&(~converged)&(m1>errTol)
	oldC=C;
	% Determining the denominator from the previous iteration	
	oldD2=D2; oldN2=N2;
	D2=freqs(A2,1,w);	
	N2=freqs(B2,1,w);
	
	if k==1,
           H2=freqs(B2,A2,w);
           E2=Htf./H2;
	end
	
	if oldm1<m1, 
	   %peso=peso*2;
	   %peso=peso;
	else
	   %peso=peso/2;
	end;

	
        
        PesosDeErro=PesosDeErro.*(1+peso*abs(log(abs(E2)))./max(log(abs(E2))));	
	PesosDeErro = PesosDeErro./max(PesosDeErro);
        
        iPesosDeErro = find(PesosDeErro<1e-5);
        PesosDeErro(iPesosDeErro)=ones(size(iPesosDeErro))*1e-5;
		 	
        %if m1<5	
       	   mD2=imH./D2.*PesosDeErro;
	%else
        %  mD2=imH.*PesosDeErro;
 	%end

	% DC constraint
	mmm=max(abs(mD2));
	mD2(1)=1000*mmm(1);
	
	A1=diag(mD2)*oldA1;
	b1=diag(mD2)*oldb1;
			
	A=[real(A1); imag(A1)];
	B=[real(b1); imag(b1)];
		
	if (find(~finite(A)))
		disp('INFINITOS!!!');
	end		
	
	[Q,R]=qr(A,0);
	BB=Q'*B;
	
	C=R\BB;
	
	B3=B2;  	% DEBUG
        A3=A2;		%

	B2=C(nb+1:-1:1);
	A2=[C(nb+na+1:-1:nb+2); 1];

	H2=freqs(B2,A2,w);
	E2=Htf./H2;
	oldm1=m1;
	m1=max(abs(20*log10(abs(E2))));
	m2=mean(abs(20*log10(abs(E2))));

        m1_acum=[m1_acum m1];
        m2_acum=[m2_acum m2];
        
        i_min_m1 = find(m1_acum==min(m1_acum));
        
        if(i_min_m1(length(i_min_m1))==length(m1_acum))
           bestA=A2;
           bestB=B2;
        end

	%
	%	Some graphical output (partially stolen from qr3_1)
	%	
	if ~exist('f1') & (show==1), 
		f1=figure; 
		ax1_f1=axes('Position',[0.05 0.275+0.5 0.425 0.180]);
		ax2_f1=axes('Position',[0.05 0.05+0.5 0.425 0.180]);
		ax3_f1=axes('Position',[0.05 0.3 0.425 0.180]);
		ax4_f1=axes('Position',[0.05 0.05 0.425 0.180]);
		ax5_f1=axes('Position',[0.05+0.5 0.275+0.5 0.425 0.180]);
		ax6_f1=axes('Position',[0.05+0.5 0.05+0.5 0.425 0.180]);
		ax7_f1=axes('Position',[0.05+0.5 0.05 0.425 0.430]);			
	end;		
	
	if show==1
		figure(f1);
		if(k==3), set(f1,'visible','off'); end;
		set(f1,'Units','normal','Number','off','Name','l2minit3 - output','Menu','none');

      axes(ax1_f1);
      %cla;
      plot(log10(omega),20*log10(abs(Htf)),log10(omega),20*log10(abs(H2)));	
      set(ax1_f1,'FontSize',8);
      title('Magnitude');

      axes(ax2_f1);
      %cla;
      plot(log10(omega),20*log10(abs(E2)));		
      set(ax2_f1,'FontSize',8);
      title('Error Magnitude');				

      axes(ax3_f1);
      %cla;		
      semilogy(1:length(m1_acum),m1_acum,1:length(m2_acum),m2_acum);
      set(ax3_f1,'FontSize',8);
      title('Maximum Magnitude Error');

      axes(ax4_f1);
      %cla;
      plot(log10(omega),log10(PesosDeErro));
      set(ax4_f1,'FontSize',8);
      title('Error weighting');
	
      axes(ax5_f1);
      %cla;
      plot(log10(omega),180/pi*unwrap(angle([Htf H2])));		
      set(ax5_f1,'FontSize',8);
      title('Phase');
	
      axes(ax6_f1);	
      %cla;
      plot(log10(omega),180/pi*unwrap(angle(E2)));
      set(ax6_f1,'FontSize',8);
      title('Phase error');
		
      axes(ax7_f1);	
      %cla;
      Poles=roots(A2);
      Zeros=roots(B2);
      plot(real(Poles),imag(Poles),'x',real(Zeros),imag(Zeros),'o');
      grid;
      set(ax7_f1,'FontSize',8);
      title('Poles and Zeros');
	

      if(k==1), set(f1,'visible','on'); end;
      drawnow;
   end;		
	
	s=max(abs((C-oldC)./(abs(C)+abs(oldC))));	
		
	%
	%	Some command line output
	%
	if show==1
		disp(['Iteration ' num2str(k) ': maximum error ' num2str(m1) 'dB average error:' num2str(m2) 'dB.']);	
		disp(['s=max(abs((C-oldC)./(abs(C)+abs(oldC))))=' num2str(s) '.']);
	end		

	if((s<tol)|(abs(m1)<errTol))
		converged=1;
	else
		converged=0;
	end;
	
	k=k+1;
%	pause;
end;	

if exist('bestB')
   B2=bestB(:).'; A2=bestA(:).';
else
   B2=B2(:).'; A2=A2(:).'
end

B2=B2/A2(1);
A2=A2/A2(1);



%
%	If the least-squares did it then show the graphs that otherwise wouldn't show up
%
if (k==1) & (show==1), 
	f1=figure; 
	ax1_f1=axes('Position',[0.05 0.275+0.5 0.425 0.180]);
	ax2_f1=axes('Position',[0.05 0.05+0.5 0.425 0.180]);
	ax3_f1=axes('Position',[0.05 0.3 0.425 0.180]);
	ax4_f1=axes('Position',[0.05 0.05 0.425 0.180]);
	ax5_f1=axes('Position',[0.05+0.5 0.275+0.5 0.425 0.180]);
	ax6_f1=axes('Position',[0.05+0.5 0.05+0.5 0.425 0.180]);
	ax7_f1=axes('Position',[0.05+0.5 0.05 0.425 0.430]);			
				
	figure(f1);
	if(k==3), set(f1,'visible','off'); end;
	set(f1,'Units','normal','Number','off','Name','l2minit3 - output','Menu','none');

	axes(ax1_f1);
	%cla;
	plot(log10(omega),20*log10(abs(Htf)),log10(omega),20*log10(abs(H2)));	
	set(ax1_f1,'FontSize',8);
	title('Magnitude');

	axes(ax2_f1);
	%cla;
	plot(log10(omega),20*log10(abs(E2)));		
	set(ax2_f1,'FontSize',8);
	title('Error Magnitude');				

	axes(ax3_f1);
	%cla;		
	%semilogy(Macum);
	set(ax3_f1,'FontSize',8);
	title('Maximum Magnitude Error');
	
	axes(ax4_f1);
	%cla;
	%semilogy(Vacum);
	set(ax4_f1,'FontSize',8);
	title('Mean Magnitude Error');
	
	
	axes(ax5_f1);
	%cla;
	plot(log10(omega),180/pi*unwrap(angle([Htf H2])));		
	set(ax5_f1,'FontSize',8);
	title('Phase');
	
	axes(ax6_f1);	
	%cla;
	plot(log10(omega),180/pi*unwrap(angle(E2)));
	set(ax6_f1,'FontSize',8);
	title('Phase error');
		
	axes(ax7_f1);	
	%cla;
	Poles=roots(A2);
	Zeros=roots(B2);
	plot(real(Poles),imag(Poles),'x',real(Zeros),imag(Zeros),'o');
	grid;
	set(ax7_f1,'FontSize',8);
	title('Poles and Zeros');
end	



